@@ -2,6 +2,8 @@ import base64
 import json
 import time
 import logging
+import signal
+from functools import partial
 import traceback
 from datetime import datetime
 import os
@@ -439,7 +441,11 @@ def start_server():
     access_file_handler.setFormatter(formatter)
     access_logger.addHandler(access_file_handler)
 
-    app.listen(10101)
+    server = tornado.httpserver.HTTPServer(app)
+    server.listen(10101)
+
+    signal.signal(signal.SIGTERM, partial(sig_handler, server))
+    signal.signal(signal.SIGINT, partial(sig_handler, server))
 
     return app
 
@@ -463,5 +469,29 @@ def main():
         logging.getLogger('tornado.application').error(traceback.format_exc())
 
 
+def sig_handler(server, sig, frame):
+    io_loop = tornado.ioloop.IOLoop.instance()
+    MAX_WAIT_SECONDS_BEFORE_SHUTDOWN = 3
+
+    def stop_loop(deadline):
+        now = time.time()
+        if now < deadline and (io_loop._callbacks or io_loop._timeouts):
+            logging.info('Waiting for next tick')
+            io_loop.add_timeout(now + 1, stop_loop, deadline)
+        else:
+            io_loop.stop()
+            logging.info('Shutdown finally')
+
+    def shutdown():
+        logging.info('Stopping http server')
+        server.stop()
+        logging.info('Will shutdown in %s seconds ...',
+                     MAX_WAIT_SECONDS_BEFORE_SHUTDOWN)
+        stop_loop(time.time() + MAX_WAIT_SECONDS_BEFORE_SHUTDOWN)
+
+    logging.warning('Caught signal: %s', sig)
+    io_loop.add_callback_from_signal(shutdown)
+
+
 if __name__ == '__main__':
     main()
